/*
 * JBoss, Home of Professional Open Source
 * Copyright 2011, Red Hat Middleware LLC, and others contributors as indicated
 * by the @authors tag. All rights reserved.
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but WITHOUT A
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License,
 * v.2.1 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 */
package org.jboss.jbw2011.keynote.demo.drools;

import java.util.concurrent.TimeUnit ;

import org.drools.ClockType ;
import org.drools.KnowledgeBase ;
import org.drools.KnowledgeBaseConfiguration ;
import org.drools.KnowledgeBaseFactory ;
import org.drools.builder.KnowledgeBuilder ;
import org.drools.builder.KnowledgeBuilderFactory ;
import org.drools.builder.ResourceType ;
import org.drools.conf.EventProcessingOption ;
import org.drools.io.ResourceFactory ;
import org.drools.runtime.KnowledgeSessionConfiguration ;
import org.drools.runtime.StatefulKnowledgeSession ;
import org.drools.runtime.conf.ClockTypeOption ;
import org.drools.runtime.rule.WorkingMemoryEntryPoint ;
import org.drools.time.SessionPseudoClock ;
import org.jboss.jbw2011.keynote.demo.drools.model.Tweet ;

public class RulesService
{
    private final StatefulKnowledgeSession session ;
    private final WorkingMemoryEntryPoint entryPoint  ;
    private boolean terminated ;
    
    public RulesService(final String ruleset, final TweetProcessor tweetProcessor)
        throws RulesServiceException
    {
        final KnowledgeBase knowledgeBase = createKnowledgeBase(ruleset) ;
        if (knowledgeBase == null)
        {
            throw new RulesServiceException("Could not locate ruleset: " + ruleset) ;
        }
        else
        {
            session = createKnowledgeSession(knowledgeBase) ;
            session.setGlobal("tweetProcessor", tweetProcessor) ;
            
            entryPoint = session.getWorkingMemoryEntryPoint( "twitter" );
            if (entryPoint == null)
            {
                session.dispose() ;
                throw new RulesServiceException("Rules do not contain the twitter entry point") ;
            }
        }
    }
    
    public void processTweet(final Tweet tweet)
        throws RulesServiceException
    {
        if (terminated)
        {
            throw new RulesServiceException("Session terminated") ;
        }
        
        final SessionPseudoClock sessionClock = session.getSessionClock() ;

        final long currentTime = sessionClock.getCurrentTime() ;
        final long tweetTimestamp = tweet.getTimestamp() ;
        if (tweetTimestamp > currentTime)
        {
            sessionClock.advanceTime(tweetTimestamp - currentTime, TimeUnit.MILLISECONDS) ;
        }

        entryPoint.insert(tweet) ;
        sessionClock.advanceTime(0, TimeUnit.SECONDS) ;
        session.fireAllRules() ;
    }
    
    public void dispose()
    {
        session.dispose() ;
        terminated = true ;
    }
    
    private static KnowledgeBase createKnowledgeBase(final String rulesFile)
    {
        // Parses and compiles the rules file
        final KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder() ;
        kbuilder.add(ResourceFactory.newClassPathResource( rulesFile ), ResourceType.DRL) ;
        
        // Configures the Stream mode
        KnowledgeBaseConfiguration conf = KnowledgeBaseFactory.newKnowledgeBaseConfiguration() ;
        conf.setOption(EventProcessingOption.STREAM) ;
        
        // Creates the knowledge base and adds the rules
        KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase(conf) ;
        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages()) ;
        return kbase;
    }

    private static StatefulKnowledgeSession createKnowledgeSession(final KnowledgeBase kbase)
    {
        final KnowledgeSessionConfiguration ksconf = KnowledgeBaseFactory.newKnowledgeSessionConfiguration() ;
        ksconf.setOption(ClockTypeOption.get(ClockType.PSEUDO_CLOCK.getId())) ;
        final StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession(ksconf, null) ;
        return ksession ;
    }
}
